#
#        アルゴリズム5.2.1によるシミュレーション
#
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss

NSamples = 10_000
L_Ns = [5, 10, 20, 50, 100, 200, 500]
L_f = [0.1, 0.25, 0.40]  
v_alpha = 0.05                                #   有意水準
a = 3                                         #   ３水準
    
powers = np.empty((len(L_f), len(L_Ns)))
for i_f, v_f in enumerate(L_f):
    for j_N, N in enumerate(L_Ns):
        df_A = a - 1
        df_S_A = a * (N -1)
        lmbd = a * N * (v_f ** 2)              #   lambdaの値
        F_a = ss.f.ppf(0.95, df_A, df_S_A)     #   限界値  
        
        count, i = 0, 1                        #   初期値設定
        while True:
            v1 = ss.ncx2.rvs(df_A, lmbd)  
            v2 = ss.chi2.rvs(df_S_A)  
            F_v = (v1/df_A) / (v2/df_S_A)  
            if F_a < F_v:                       #   有意？
                count += 1
            i += 1
            if i > NSamples:                    #   設定条件でのシミュレーション終了？
                break
        power = count / NSamples                #   検定力
        powers[i_f][j_N] = power
                
        print('i_f =', i_f, '   j_N =', j_N)
        print('power = ', power)
        
for i in range(len(L_f)):
    print(powers[i])
#
#         シミュレーション結果のグラフ描画
#
v_log_x = np.log(L_Ns)
L_log_x = ['{}'.format(v) for v in L_Ns]
for j in range(len(L_f)):
    plt.plot(v_log_x, powers[j], label = 'f = {}'.format(L_f[j]))
plt.xticks(v_log_x, L_log_x)
plt.xlabel('N', fontsize = 16)
plt.yticks([0, 0.5, 0.95, 1])
plt.ylabel('Power', fontsize = 16)
plt.hlines(1.0, np.log(5), np.log(500), lw = 1)
plt.hlines(0.95, np.log(5), np.log(500), lw = 1)
plt.hlines(0.5, np.log(5), np.log(500), lw = 1)
plt.hlines(0, np.log(5), np.log(500), lw = 1)
plt.legend(loc = 'lower right')
plt.title('Effect Size f, Sample Size N, and Power', fontsize = 18)
plt.tight_layout()
plt.show()
